<li class="nav-item">
    <a class="nav-link" aria-current="page" href="{% url 'metenox:index' %}">Moons</a>
</li>
<li class="nav-item">
    <a class="nav-link" aria-current="page" href="{% url 'metenox:metenoxes' %}">Metenoxes</a>
</li>
<li class="nav-item">
    <a class="nav-link" aria-current="page" href="{% url 'metenox:corporations' %}">Corporations</a>
</li>
<li>
    <a class="nav-link" aria-current="page" href="{% url 'metenox:prices' %}">Prices</a>
</li>
