{% load metenox %}

<p class="text-muted">
    The est. value reprensents the monthly (30 days) harvest of a metenox moon goo sold for jita buy price.
    <br>
    The monthly fuel price of approx {{ monthly_fuel_price|formatisk }} isn't included there.
    Click the button on the left side for the estimated profit
</p>
