{% extends "metenox/base.html" %}
{% load metenox %}
{% load evelinks %}

{% block content %}

    <table class="table">
        <thead>
            <tr>
                <th>Icon</th>
                <th>Name</th>
                <th>Price</th>
            </tr>
        </thead>
        <tbody>
        {% for goo_price in goo_prices %}
            <tr>
                <th><img src="{{ goo_price.eve_type.id|type_icon_url:64 }}" alt="{{ goo_price.moon_goo.name }} icon"></th>
                <th>{{ goo_price.eve_type.name }}</th>
                <th>{{ goo_price.price | formatisk }} ISK</th>
            </tr>
        {% endfor %}
        </tbody>
    </table>

{% endblock %}
