{% url "metenox:moon_details" moon.pk as new_page_url %}
{% include "metenox/modals/content_base.html" with modal_content_id="modalMoonDetailsContent" modal_body_url="metenox/partials/moon_details.html" modal_title="Moon" new_page_url=new_page_url %}
